from restcountries.client import Client
from restcountries.models import Country as CountryModel


class Country(Client):
    """Country operations."""

    def __init__(self, **kwargs):
        super(Country, self).__init__(**kwargs)

    def _country_list(self, url, params=None):
        result = self._get(url, params=params)
        return CountryModel._parse_list(result)

    def _country_get(self, url, params=None):
        result = self._get(url, params=params)
        return CountryModel._parse(result)

    def all(self):
        """
        Get all list of countries.

        :return [List]: Country list.
        """
        url = "/all"
        return self._country_list(url)

    def name(self, name, full_text=False):
        """
        Search by country name. It can be the native name or partial name.

        :param name: Country name.
        :param full_text: Search by country full name.
        :return [List]: Country list.
        """
        url = "/name/%s" % name
        params = {"fullText": full_text}
        return self._country_list(url, params=params)

    def code(self, code):
        """
        Search by ISO 3166-1 2-letter or 3-letter country code.

        :param code: Country code.
        :return [List]: Country list.
        """
        url = "/alpha/%s" % code
        return self._country_get(url)

    def code_list(self, code_list):
        """
        Search by list of ISO 3166-1 2-letter or 3-letter country codes.

        :param code_list: Country code list.
        :return [List]: Country list.
        """
        url = "/alpha"
        codes = ";".join(code_list)
        params = {"codes": codes}
        return self._country_list(url, params=params)

    def currency(self, currency_code):
        """
        Search by ISO 4217 currency code.

        :param currency_code: Currency code.
        :return [List]: Country list.
        """
        url = "/currency/%s" % currency_code
        return self._country_list(url)

    def language(self, language_code):
        """
        Search by ISO 4217 currency code.

        :param language_code: Language code.
        :return [List]: Country list.
        """
        url = "/lang/%s" % language_code
        return self._country_list(url)

    def capital(self, capital):
        """
        Search by capital city.

        :param capital: Capital city.
        :return [List]: Country list.
        """
        url = "/capital/%s" % capital
        return self._country_list(url)

    def calling_code(self, calling_code):
        """
        Search by calling code.

        :param calling_code: Calling code.
        :return [List]: Country list.
        """
        url = "/callingcode/%s" % calling_code
        return self._country_list(url)

    def region(self, region):
        """
        Search by region: Africa, Americas, Asia, Europe, Oceania.

        :param region: Region.
        :return [List]: Country list.
        """
        url = "/region/%s" % region
        return self._country_list(url)

    def regional_bloc(self, regional_bloc):
        """
        Search by regional bloc:
            EU (European Union)
            EFTA (European Free Trade Association)
            CARICOM (Caribbean Community)
            PA (Pacific Alliance)
            AU (African Union)
            USAN (Union of South American Nations)
            EEU (Eurasian Economic Union)
            AL (Arab League)
            ASEAN (Association of Southeast Asian Nations)
            CAIS (Central American Integration System)
            CEFTA (Central European Free Trade Agreement)
            NAFTA (North American Free Trade Agreement)
            SAARC (South Asian Association for Regional Cooperation)

        :param regional_bloc: Regional bloc.
        :return [List]: Country list.
        """
        url = "/regionalbloc/%s" % regional_bloc
        return self._country_list(url)



import six


class RestCountryException(Exception):
    """RestCountry exception"""

    def __init__(self, message, **kwargs):
        self.message = six.text_type(message) if message else "Unknown error"
        super(RestCountryException, self).__init__(message, **kwargs)

    def __str__(self):
        return self.message


class NotFound(RestCountryException):
    message = six.text_type("Not Found")



from restcountries.expections import NotFound


class ResultSet(list):
    """A list like object that holds results from a RestCountries API query."""


class Model(object):
    _not_found_error_class = NotFound

    def __init__(self, **kwargs):
        self._repr_values = {"id": "Id"}

    @classmethod
    def _parse(cls, data, sub_item=False):
        data = data or {}
        if not data and not sub_item:
            raise cls._not_found_error_class()

        instance = cls() if data else None
        for key, value in data.items():
            if type(value) == str:
                value = value.strip()
            setattr(instance, key, value)
        return instance

    @classmethod
    def _parse_list(cls, data, sub_item=False):
        """Parse a list of JSON objects into a result set of model instances."""
        results = ResultSet()
        data = data or []
        for obj in data:
            if obj:
                results.append(cls._parse(obj, sub_item=sub_item))
        return results

    def __repr__(self):
        items = filter(lambda x: x[0] in self._repr_values.keys(), vars(self).items())
        state = ['%s: %s' % (self._repr_values[k], repr(v)) for (k, v) in items]
        return '<%s: %s >' % (self.__class__.__name__, ', '.join(state))


class Country(Model):

    def __init__(self, **kwargs):
        super(Country, self).__init__(**kwargs)
        self._repr_values = {"name": "Name", "alpha2Code": "Code"}

    @classmethod
    def _parse(cls, data, sub_item=False):
        country = super(Country, cls)._parse(data, sub_item=sub_item)

        if hasattr(country, "currencies"):
            country.currencies = Currency._parse_list(country.currencies, sub_item=True)

        if hasattr(country, "languages"):
            country.languages = Language._parse_list(country.languages, sub_item=True)

        if hasattr(country, "regionalBlocs"):
            country.regional_blocs = RegionalBloc._parse_list(country.regionalBlocs, sub_item=True)

        return country


class Currency(Model):

    def __init__(self, **kwargs):
        super(Currency, self).__init__(**kwargs)
        self._repr_values = {"name": "Name", "code": "Code"}


class Language(Model):

    def __init__(self, **kwargs):
        super(Language, self).__init__(**kwargs)
        self._repr_values = {"name": "Name"}


class RegionalBloc(Model):

    def __init__(self, **kwargs):
        super(RegionalBloc, self).__init__(**kwargs)
        self._repr_values = {"name": "Name"}



from restcountries.country import Country


class RestCountries(object):
    """RestCountries API"""
    BASE_URL = "http://restcountries.eu/rest"
    VERSION = "v2"

    def __init__(self, **kwargs):
        """
        RestCountries Api Instance Constructor

        :param kwargs:
        """
        self.base_url = "%s/%s" % (self.BASE_URL, self.VERSION)

    @property
    def _country(self):
        return Country(api=self)

    def all(self):
        return self._country.all()

    def name(self, name, full_text=False):
        return self._country.name(name, full_text=full_text)

    def code(self, code):
        return self._country.code(code)

    def code_list(self, code_list):
        return self._country.code_list(code_list)

    def currency(self, currency):
        return self._country.currency(currency)

    def language(self, language_code):
        return self._country.language(language_code)

    def capital(self, capital):
        return self._country.capital(capital)

    def calling_code(self, calling_code):
        return self._country.calling_code(calling_code)

    def region(self, region):
        return self._country.region(region)

    def regional_bloc(self, regional_bloc):
        return self._country.regional_bloc(regional_bloc)



import requests

from restcountries.expections import RestCountryException


class Client(object):
    """
    RestCountries Client

    HTTP connections to and communication with the RestCountries API.
    """

    def __init__(self, api, **kwargs):
        self.api = api

    def _request(self, url, method, params=None, data=None, **kwargs):
        url = "%s%s" % (self.api.base_url, url)
        headers = kwargs.pop("headers", {})

        try:
            response = requests.request(method, url, params=params, data=data, headers=headers, **kwargs)
        except Exception as e:
            raise RestCountryException("Connection error: %s" % e)

        try:
            if not self._is_2xx(response.status_code):
                error = response.json().get("message")
                raise RestCountryException(error)
            result = response.json()
        except ValueError as e:
            result = None
        return result

    def _get(self, url, params=None, **kwargs):
        return self._request(url, "get", params=params, **kwargs)

    def _post(self, url, data=None, **kwargs):
        return self._request(url, "post", data=data, **kwargs)

    def _delete(self, url, **kwargs):
        return self._request(url, "delete", **kwargs)

    def _put(self, url, data=None, **kwargs):
        return self._request(url, "put", data=data, **kwargs)

    @staticmethod
    def _is_1xx(status_code):
        return 100 <= status_code <= 199

    @staticmethod
    def _is_2xx(status_code):
        return 200 <= status_code <= 299

    @staticmethod
    def _is_3xx(status_code):
        return 300 <= status_code <= 399

    @staticmethod
    def _is_4xx(status_code):
        return 400 <= status_code <= 499

    @staticmethod
    def _is_5xx(status_code):
        return 500 <= status_code <= 599



#!/usr/bin/env python
# coding=utf-8

import uuid

from setuptools import setup, find_packages

try:  # for pip >= 10
    from pip._internal.req import parse_requirements
except ImportError:  # for pip <= 9.0.3
    from pip.req import parse_requirements

install_requirements = parse_requirements('requirements.txt', session=uuid.uuid1())
requirements = [str(req.req) for req in install_requirements]

setup(
    name="restcountries",
    version="1.0.0",
    description="A Python client for the RestCountries API.",
    license="MIT",
    author="Yakup Adaklı",
    author_email="yakup.adakli@gmail.com",
    url="http://github.com/yakupadakli/restcountries.git",
    packages=find_packages(exclude=["tests"]),
    install_requires=requirements,
    keywords="restcountries library countries",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Topic :: Software Development :: Libraries",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
    ],
    zip_safe=True,
)



# coding=utf-8

# Unsplash
# Copyright 2018 Yakup Adaklı
# See LICENSE for details.

"""A library that provides a Python wrapper to the RestCountries API"""

__version__ = "1.0.0b1"
__author__ = "Yakup Adaklı"
__license__ = "MIT"
